/*
 * Created on 2004-10-6
 *
 */
package org.fmi.bdi.db;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;


import org.fmi.bdi.db.domains.IMessageType;
import org.fmi.bdi.db.domains.IRelationType;
import org.fmi.bdi.document.GenericBDIDocumentWrapper;
import org.fmi.bdi.helpers.AppContext;
import org.fmi.bdi.helpers.BDIException;
import org.fmi.bdi.helpers.DBConnectionPool;
import org.fmi.bdi.helpers.DocumentType;
import org.fmi.bdi.helpers.SourceSystem;
import org.fmi.bdi.helpers.Utils;
import org.w3c.dom.Document;

/**
 * @author IvoJekov
 *
 *	Wraps all Db activities
 */
public class DBHelper
{

	/**
	 * Initialize supplied GenericBDIDocumentWrapper
	 * 
	 * @param sourceSystemId  	should be T_SOURCE_SYSTEM.SOURCE_SYSTEM_ID
	 * @param docTypeId			should be TD_DOCUMENT_TYPE.DOCUMENT_TYPE_ID
	 * @param doc				document wrapper to be populated
	 * @throws BDIException		if DB error occurs or 
	 * 							relation Source System/Document Type is not found
	 */
	public static void populateDocumentInfo(int sourceSystemId,
											int docTypeId,
											GenericBDIDocumentWrapper doc) throws BDIException
	{
		AppContext app = AppContext.getInstance();
		DBConnectionPool pool = app.getDbConPool();
		
		String sql = " select 	ss.\"SOURCE_SYSTEM_ID\", \n"
					+ "			ss.\"NAME\", \n"
					+ "			ss.\"VERSION\", \n"
					+ "			doc.\"DOCUMENT_ID\", \n"
					+ "			doc.\"VALIDATION_SOURCE\", \n"
					+ "			t.\"TRANSFORMER_ID\", \n"
					+ "			t.\"JAVA_CLASS_NAME\", \n"
					+ "			vd.\"VALIDATION_TYPE_ID\", \n"
					+ "			vd.\"CODE\" as \"VAL_CODE\", \n"
					+ "			dt.\"DOCUMENT_TYPE_ID\", \n"
					+ "			dt.\"NAME\" as \"DOC_TYPE_NAME\", \n"
					+ "			dt.\"CODE\" as \"DOC_CODE\" \n"
					+ "	from 	\"T_SOURCE_SYSTEM\" ss,  \n"
					+ "			\"T_DOCUMENT\" doc,  \n"
					+ "			\"T_TRANSFORMER\" t, \n"
					+ "			\"TD_VALIDATION_TYPE\" vd, \n"
					+ "			\"TD_DOCUMENT_TYPE\" dt \n"
					+ "	where	doc.\"SOURCE_SYSTEM_ID\" = ss.\"SOURCE_SYSTEM_ID\" \n"
					+ "	and		doc.\"DOCUMENT_TYPE_ID\" = dt.\"DOCUMENT_TYPE_ID\" \n"
					+ "	and		doc.\"TRANSFORMER_ID\" = t.\"TRANSFORMER_ID\" \n"
					+ "	and		doc.\"VALIDATION_TYPE_ID\" = vd.\"VALIDATION_TYPE_ID\" \n"
					+ "	and		doc.\"SOURCE_SYSTEM_ID\" = ? \n"
					+ "	and		doc.\"DOCUMENT_TYPE_ID\" = ? \n";
		
		Connection con = pool.getConnection();
		PreparedStatement ps = null;
		ResultSet rs = null;
		try
		{
			ps = con.prepareStatement(sql);
			ps.setInt(1, sourceSystemId);
			ps.setInt(2, docTypeId);
			
			rs = ps.executeQuery();
			
			if (rs.next())
			{
				// populate document info
				String ssName = rs.getString("NAME");
				String ssVersion = rs.getString("VERSION");
				SourceSystem ss = new SourceSystem(sourceSystemId, ssName, ssVersion);
				doc.setSourceSystem(ss);
				
				doc.setDocumentID(rs.getInt("DOCUMENT_ID"));
				doc.setValidationSource(rs.getString("VALIDATION_SOURCE"));
				
				doc.setTransformerID(rs.getInt("TRANSFORMER_ID"));
				doc.setTransformerJavaClass(rs.getString("JAVA_CLASS_NAME"));
				doc.setValidationTypeID(rs.getInt("VALIDATION_TYPE_ID"));
				doc.setValidationTypeCode(rs.getString("VAL_CODE"));
				
				String docCode = rs.getString("DOC_CODE");
				String docName = rs.getString("DOC_TYPE_NAME");
				DocumentType docType = new DocumentType(docTypeId, docCode, docName);
				doc.setDocType(docType);
			}
			else
			{
				throw new BDIException("Transformation for this Source System/Document Type has not been implemented yet.");
				
			}
		}
		catch (SQLException e)
		{
			throw new BDIException("Unexpected error occured: " + e.getMessage(),
									e);
		}
		finally
		{
			try
			{
				if (rs != null)
					rs.close();
				if (ps != null)
					ps.close();
			}
			catch (SQLException e)
			{
				throw new BDIException("Unexpected error occured: " + e.getMessage(),
						e);
			}
			finally
			{
				pool.release(con);
			}
		}
	}
	
	/**
	 * Stores incoming/outgoing documents and corresponding message relations
	 * 
	 * @param dbCon 		connection to DB
	 * @param doc			document wrapper
	 * @return				the ID of newly generated record in T_MESSAGE table
	 * @throws BDIException
	 */
	public static int saveMessage(Connection dbCon, 
									GenericBDIDocumentWrapper doc) throws BDIException
	{
		CallableStatement cs = null;
		try
		{
			cs = dbCon.prepareCall("{?= call store_message(?, ?, ?, ?, ?)}");
			cs.registerOutParameter(1, Types.INTEGER);
			cs.setInt(2,doc.getDocumentID());
			
			// if transformed document is null
			// the incoming should be stored
			Document docToStore = doc.getTransformedDocument();
			if (docToStore == null)
			{
				// store incoming
				docToStore = doc.getIncomingDocument();
				
				cs.setInt(3,IMessageType.INCOMING);
				cs.setString(4, Utils.xmlToString(docToStore));
				cs.setNull(5, Types.INTEGER);
				cs.setNull(6, Types.INTEGER);
			}
			else
			{
				// store transformed document along with corresponding relations
				cs.setInt(3, IMessageType.INTERNAL);
				cs.setString(4, Utils.xmlToString(docToStore));
				cs.setLong(5, doc.getIncomingMessageID());
				cs.setInt(6, IRelationType.TRANSFORMED);
			}
			
			cs.execute();
			
			return cs.getInt(1);
			
		}
		catch (SQLException e)
		{
			throw new BDIException("Unexpected error occured: " + e.getMessage(),
					e);
		}
		finally
		{
			try
			{
				if (cs != null)
					cs.close();
			}
			catch (SQLException e)
			{
				throw new BDIException("Unexpected exception occured: " + e.getMessage(),
						e);
			}
		}
	}
	
	/**
	 * Return all source systems
	 * 
	 * @return	List of SourceSystem objects
	 * @throws BDIException
	 */
	public static List getSourceSystems() throws BDIException
	{
		List sourceSystems = new ArrayList();
		
		String sql = " select 	ss.\"SOURCE_SYSTEM_ID\", \n"
			+ "			ss.\"NAME\", \n"
			+ "			ss.\"VERSION\" \n"
			+ "	from 	\"T_SOURCE_SYSTEM\" ss  \n";

		AppContext app = AppContext.getInstance();
		DBConnectionPool pool = app.getDbConPool();
		Connection con = pool.getConnection();
		PreparedStatement ps = null;
		ResultSet rs = null;
		try
		{
			ps = con.prepareStatement(sql);
			
			rs = ps.executeQuery();
			while (rs.next())
			{
				// populate Source System info
				int ssId = rs.getInt("SOURCE_SYSTEM_ID");
				String ssName = rs.getString("NAME");
				String ssVersion = rs.getString("VERSION");
				SourceSystem ss = new SourceSystem(ssId, ssName, ssVersion);
				
				sourceSystems.add(ss);
			}
		}
		catch (SQLException e)
		{
			throw new BDIException("Unexpected error occured: " + e.getMessage(),
					e);
		}
		finally
		{
			try
			{
				if (rs != null)
					rs.close();
				if (ps != null)
					ps.close();
			}
			catch (SQLException e)
			{
				throw new BDIException("Unexpected error occured: " + e.getMessage(),
						e);
			}
			finally
			{
				pool.release(con);
			}
		}
		
		return sourceSystems;
	}
	
	/**
	 * get all DocumentTypes
	 * 
	 * @return a List of DocumentType objects
	 * @throws BDIException
	 */
	public static List getDocumentTypes() throws BDIException
	{
		List documentTypes = new ArrayList();
		
		String sql = " select 	dt.\"DOCUMENT_TYPE_ID\", \n"
			+ "			dt.\"NAME\", \n"
			+ "			dt.\"CODE\" \n"
			+ "	from 	\"TD_DOCUMENT_TYPE\" dt  \n";

		AppContext app = AppContext.getInstance();
		DBConnectionPool pool = app.getDbConPool();
		Connection con = pool.getConnection();
		PreparedStatement ps = null;
		ResultSet rs = null;
		try
		{
			ps = con.prepareStatement(sql);
			
			rs = ps.executeQuery();
			while (rs.next())
			{
				// populate Document Type info
				int dtId = rs.getInt("DOCUMENT_TYPE_ID");
				String name = rs.getString("NAME");
				String code = rs.getString("CODE");
				DocumentType dt = new DocumentType(dtId, code, name);
				
				documentTypes.add(dt);
			}
		}
		catch (SQLException e)
		{
			throw new BDIException("Unexpected error occured: " + e.getMessage(),
					e);
		}
		finally
		{
			try
			{
				if (rs != null)
					rs.close();
				if (ps != null)
					ps.close();
			}
			catch (SQLException e)
			{
				throw new BDIException("Unexpected error occured: " + e.getMessage(),
						e);
			}
			finally
			{
				pool.release(con);
			}
		}
		
		return documentTypes;
	}
	
	/**
	 * Return document as STring
	 * 
	 * @return the message as String
	 * @throws BDIException
	 */
	public static String getMessage(int msgId) throws BDIException
	{
		String sql = " select 	m.\"MESSAGE_VALUE\""
			+ "	from 	\"T_MESSAGE\" m  \n"
			+ "	where 	m.\"MESSAGE_ID\" = ?  \n";

		String msgVal = null;
		
		AppContext app = AppContext.getInstance();
		DBConnectionPool pool = app.getDbConPool();
		Connection con = pool.getConnection();
		PreparedStatement ps = null;
		ResultSet rs = null;
		try
		{
			ps = con.prepareStatement(sql);
			ps.setInt(1, msgId);
			rs = ps.executeQuery();
			if (rs.next())
			{
				// retrieve message value
				msgVal = rs.getString("MESSAGE_VALUE");
			}
			else 
			{
				throw new BDIException("Message with ID: " + msgId + " not found.");
			}
		}
		catch (SQLException e)
		{
			throw new BDIException("Unexpected error occured: " + e.getMessage(),
					e);
		}
		finally
		{
			try
			{
				if (rs != null)
					rs.close();
				if (ps != null)
					ps.close();
			}
			catch (SQLException e)
			{
				throw new BDIException("Unexpected error occured: " + e.getMessage(),
						e);
			}
			finally
			{
				pool.release(con);
			}
		}
		
		return msgVal;
	}
	
}
