package org.fmi.bdi.helpers;

import java.io.File;
import java.util.Properties;


/**
 * @author IvoJekov
 * 
 * Used to handle application initialization parameters
 */
public class AppContext
{
	// database init parameter names
	public static String PARAM_NAME_DB_DRIVER		=	"driver";
	public static String PARAM_NAME_DB_URL			=	"url";
	public static String PARAM_NAME_DB_USER			=	"userName";
	public static String PARAM_NAME_DB_PASSWORD		=	"password";
	public static String PARAM_NAME_DB_AUTO_COMMIT	=	"autoCommit";
	public static String PARAM_NAME_DB_MIN_CON		=	"minConCount";
	public static String PARAM_NAME_DB_MAX_CON		=	"maxConCount";
	
	// application init parameter names
	public static String PARAM_NAME_APP_UPLOAD_DIR		=	"uploadDir";
	public static String PARAM_NAME_APP_VAL_ROOT_DIR	=	"validationRootDir";
	
	// JAXB init parameter names
	public static String PARAM_NAME_JAXB_ORDERS_PACKAGE	=	"orderPackageName";
	
	
	// 'this' object, singelton pattern
	private static AppContext fAppContext = null;
	
	// contsins all application properties
	private Properties fProps = new Properties();
	
	// DB Pool
	private DBConnectionPool dbConPool = null;
	
	// app properties
	private File uploadDir					=	null;
	private File rootValidationDir			=	null;
	
	// used to find jaxb.properties file
	private String jaxbOrdersPackage	=	null;
	
	public static AppContext getInstance()
	{
		if (fAppContext == null)
		{
			synchronized (AppContext.class)
			{
				if (fAppContext == null)
				{
					fAppContext = new AppContext();
				}
			}
		}
		
		return fAppContext;
		
	}
	
	private AppContext()
	{
		
	}
	
	public void init(Properties initProps) throws BDIException
	{
		fProps = initProps;
		
		// init DB Connection pool
		String driver = getInitParam(PARAM_NAME_DB_DRIVER,
									null,
									true);
		String url = getInitParam(PARAM_NAME_DB_URL,
								null,
								true);
		String userName = getInitParam(PARAM_NAME_DB_USER,
										null,
										true);
		String password = getInitParam(PARAM_NAME_DB_PASSWORD,
										null,
										true);		
		boolean isAutoCommit = getInitParam(PARAM_NAME_DB_AUTO_COMMIT,
											DBConnectionPool.DEFAULT_AUTO_COMMIT,
											false);
		int minConCount = getInitParam(PARAM_NAME_DB_MIN_CON,
											DBConnectionPool.DEFAULT_MIN_CONNECTION_COUNT,
											false);
		int maxConCount = getInitParam(PARAM_NAME_DB_MAX_CON,
										DBConnectionPool.DEFAULT_MAX_CONNECTION_COUNT,
										false);
		
		dbConPool = new DBConnectionPool(driver, 
										url, 
										userName,
										password,
										isAutoCommit,
										minConCount,
										maxConCount);
		
		// init application and JAXB properties
		jaxbOrdersPackage = getInitParam(PARAM_NAME_JAXB_ORDERS_PACKAGE,
										null,
										true);
		
		uploadDir = getDir(PARAM_NAME_APP_UPLOAD_DIR,
							false);
		rootValidationDir = getDir(PARAM_NAME_APP_VAL_ROOT_DIR,
							true);
		

	}
	
	private String getInitParam(String paramName, 
								String defValue,
								boolean isMandatory) throws BDIException
	{
		String paramValue = fProps.getProperty(paramName);
		if (paramValue == null || paramValue.trim().equals(""))
		{	
			if (isMandatory)
			{	
				throw new BDIException("Initialization failed: Required parameter '" 
									+ paramName 
									+ "' is missing.");
			}
			
			paramValue = defValue;
		}
		
		return paramValue;
	}
	
	private int getInitParam(String paramName, 
							int defValue,
							boolean isMandatory) throws BDIException
	{
		String paramValue = getInitParam(paramName,
										"" + defValue,
										isMandatory);
		try
		{
			return Integer.parseInt(paramValue);
		}
		catch (NumberFormatException nfe)
		{
			
			throw new BDIException("Initialization failed: parameter '" 
					+ paramName 
					+ "' is not a number.");
		}
	}

	private boolean getInitParam(String paramName, 
								boolean defValue,
								boolean isMandatory) throws BDIException
	{
		String paramValue = getInitParam(paramName,
									"" + defValue,
									isMandatory);
		return "true".equalsIgnoreCase(paramValue);
	}
	
	private File getDir(String paramName,
						boolean mustExist) throws BDIException
	{
		String dirPath = getInitParam(paramName,
										null,
										true);
		
		File dir = new File(dirPath);
		boolean existsDir = dir.exists();
		if (!existsDir)
		{
			if (mustExist)
				throw new BDIException("Initialization failed: directory '" 
						+ dir.getPath() 
						+ "' does not exists.");
			
			dir.mkdirs();
		}
			
		
		return dir;
	}
	
	/**
	 * @return Returns the jaxbOrdersPackage.
	 */
	public String getJaxbOrdersPackage()
	{
		return jaxbOrdersPackage;
	}


	/**
	 * @return Returns the uploadDir.
	 */
	public String getUploadDir()
	{
		return uploadDir.getPath();
	}

	/**
	 * @return Returns the uploadDir.
	 */
	public String getRootValidationDir()
	{
		return rootValidationDir.getPath();
	}
	
	/**
	 * @return Returns the dbConPool.
	 */
	public DBConnectionPool getDbConPool()
	{
		return dbConPool;
	}

}
