/*
 * Created on 2004-8-22
 *
 */
package org.fmi.bdi.helpers;

/**
 * @author IvoJekov
 * 
 * This is the base class for JAXB related classes
 *
 */
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.dom.DOMResult;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;


public class BaseJAXBDoc implements Cloneable
{
	private JAXBContext fJaxbContext = null;

	protected String fContextPath = "";

	/**
	 * This is a hook for sub-classes. 
	 * <br>This implementation does nothing;
	 */
	public void unMarshal(Object xml) throws BDIException {
		if (xml != null) {
			throw new BDIException("unmarshal() not implemented!");
		}
	}

	/**
	 * this is a hook for sub-classes. <br>This implementation does nothing;
	 */
	public void marshal(OutputStream out) throws BDIException {
		if (out != null) {
			throw new BDIException("marshal() not implemented!");
		}
	}

	/**
	 * converts XML to Jaxb classes <br>create a JAXBContext capable of
	 * handling classes generated by JAXB use set/getContextPath(..) to change
	 * the package path <br>create an Unmarshaller <br>determine xml input
	 * type <br>
	 * 
	 * @return the jaxb object.
	 *  
	 */
	protected Object genericUnMarshal(Object xml) throws BDIException {
		try {
			setJaxbContext(JAXBContext.newInstance(getContextPath()));
			Unmarshaller unMarshaller = getJaxbContext().createUnmarshaller();
			unMarshaller.setValidating(true);
			
			if (xml instanceof File)
				return unMarshaller.unmarshal((File) xml);
			else if (xml instanceof InputStream)
				return unMarshaller.unmarshal((InputStream) xml);
			else if (xml instanceof Reader) {
				InputSource is = new InputSource((Reader) xml);
				return unMarshaller.unmarshal(is);
			}												
			else if (xml instanceof String) {
				Reader reader = new StringReader((String) xml);
				InputSource is = new InputSource();
				is.setCharacterStream(reader);

				return unMarshaller.unmarshal(is);
			} else
				return null;
		} catch (UnmarshalException e) {
			String msg = e.toString();
			SAXParseException saxe = null;
			if (e.getLinkedException() != null
					&& e.getLinkedException() instanceof SAXParseException)
				saxe = (SAXParseException) e.getLinkedException();

			if (saxe != null)
				msg += "Line number "
					+ saxe.getLineNumber()
					+ " Column "
					+ saxe.getColumnNumber();

			throw new BDIException("genericUnMarshal: "+ msg, e);
		} catch (Exception e) {
			throw new BDIException(
					"genericUnMarshal: Unexpected Error occured: " + e.getMessage(), e);
		}
	}

	/**
	 * writes out the doc in xml format.
	 */
	public void marshal(OutputStream out, Object object) throws BDIException {
		// create a Marshaller and marshal to a file
		try {
			Marshaller m = getJaxbContext().createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(object, out);
		} catch (Exception e) {
			throw new BDIException(
					"marshall(): Unexpected error occured: ", e);
		}
	}

	/**
	 * writes out the doc in xml format.
	 */
	public void marshal(Writer out, Object object) throws BDIException {
		// create a Marshaller and marshal to a file
		try {
			Marshaller m = getJaxbContext().createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(object, out);
		} catch (Exception e) {
			throw new BDIException(
					"marshall(): unexpected error occured: ", e);
		}
	}

	/**
	 * writes out the doc in xml format.
	 */
	public Document marshal(Object object) throws BDIException {
		// create a Marshaller and marshal to a file
		try {
			DOMResult domRes = new DOMResult();
			Marshaller m = getJaxbContext().createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(object, domRes);
			
			return (Document) domRes.getNode();
		} catch (Exception e) {
			throw new BDIException(
					"marshall(): unexpected error occured: " + e.getMessage(),
					e);
		}
	}
	
	/**
	 * Returns the jaxbContext.
	 * 
	 * @return JAXBContext
	 */
	public JAXBContext getJaxbContext() {
		return fJaxbContext;
	}

	/**
	 * Sets the jaxbContext.
	 * 
	 * @param jaxbContext
	 *            The jaxbContext to set
	 */
	public void setJaxbContext(JAXBContext jaxbContext) {
		fJaxbContext = jaxbContext;
	}

	/**
	 * Returns the contextPath.
	 * 
	 * @return String
	 */
	public String getContextPath() {
		return fContextPath;
	}

	/**
	 * Sets the contextPath.
	 * 
	 * @param contextPath
	 *            The contextPath to set
	 */
	public void setContextPath(String contextPath) {
		fContextPath = contextPath;
	}
	
	/**
	 * 
	 * @param obj
	 * @return
	 * @throws CloneNotSupportedException
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static Object clone(Object obj)
				throws CloneNotSupportedException, 
						IOException, 
						ClassNotFoundException 
	{
		Object clonedObj = null;
		ByteArrayOutputStream baOut = null;
		ObjectOutputStream oOut = null;
		ByteArrayInputStream baIn = null;
		ObjectInputStream oIn = null;

		//serialize
		baOut = new ByteArrayOutputStream();
		oOut = new ObjectOutputStream(baOut);
		oOut.writeObject(obj);
		oOut.flush();

		//deserialize
		baIn = new ByteArrayInputStream(baOut.toByteArray());
		oIn = new ObjectInputStream(baIn);
		clonedObj = oIn.readObject();
		return clonedObj;
	}

	
}
