/*
 * Created on 2004-8-2
 *
 */
package org.fmi.bdi.helpers;

import java.util.List;
import java.util.Properties;
import java.util.Vector;

import java.sql.Connection;
import java.sql.DriverManager;


/**
 * @author IvoJekov
 * 
 * Handles the details related to DB connection pooling
*/
public class DBConnectionPool
{
	// default parameter values
	public static final int DEFAULT_MIN_CONNECTION_COUNT	=	1;
	public static final int DEFAULT_MAX_CONNECTION_COUNT	=	3;
	
	public static final int DEFAULT_INCREMENT_COUNT			=	1;
		
	public static final boolean DEFAULT_AUTO_COMMIT			=	false;
	

	private int fMin;
	private int fMax;

	// stores the connection properties
	private Properties fConProps = new Properties();
	private List fConnections = new Vector();

	// the locked list is used only as a means of forcing the connections
	//  on lease to close.
	private Vector fLeasedList = new Vector();

	// monitor fields
	private int fMonitorMaxReached = 0;
	private int fMonitorInvalidConnections = 0;
	private int fConnectionsUsedCount = 0;
	
	
	protected DBConnectionPool(String driver, 
							 String url, 
							 String user, 
							 String pwd, 
							 boolean autoCommit,
							 int min, 
							 int max) throws BDIException
	{
		String m = getClass().getName() + ".DBConnectionPool()";
		
		try
		{
			fMin = min;
			fMax = max;

			Properties info = new Properties();
			fConProps.setProperty("user", user);
			fConProps.setProperty("password", pwd);
			fConProps.setProperty("driver", driver);
			fConProps.setProperty("url", url);

			fConProps.setProperty("autoCommit", "" + autoCommit);


			openPool();
		}
		catch(BDIException e)
		{
			throw e;
		}
		catch(Exception e)
		{
			throw new BDIException(m + " failed: ", e);
		}
		
	}
	
	/**
	 * Adds  a connection to the pool.<br>
	 * No validation is done in terms of min,max and increment.
	 *
	 * @throws BDIException if the connection could not be added
	 *
	 */
	protected void addConnection() throws BDIException
	{
		String m = getClass().getName() + ".addConnection()";
		
		//------------------------------------------------------------------
		// instatiate driver:: will fail if jar/zip not in class path
		// connect:: will fail if db not open or password/user details incorrect
		//------------------------------------------------------------------
		Connection con = null;

		try
		{
			Class.forName(getDriver()).newInstance();

			con = DriverManager.getConnection(getURL(), fConProps);
			con.setAutoCommit(getAutoCommit());
			
			// add the connection to the pool
			fConnections.add(con);
			
		}
		catch(Exception e)
		{
			throw new BDIException(m + "failed: ", e);
		}

	}


	/**
	 * Returns a valid connection or throws an exception
	 *
	 * @return Connection The valid connection
	 *
	 * @throws BDIException if the connection could not be obtained
	 *
	 */
	public Connection getConnection() throws BDIException
	{
		if(fConnections.size() == 0)
		{
			doIncrement();
		}

		Connection con = (Connection) fConnections.remove(0);

		if(!checkConnection(con))
		{
			return getConnection();
		}

		fLeasedList.add(con);
		fConnectionsUsedCount++;

		return con;
	}


	/**
	 * Releases the connection back to the pool.<br>
	 * The connection is pruned if the max property has been exceeded.
	 *
	 * @throws BDIException if the connection could not be released
	 *
	 */
	public void release(Connection con) throws BDIException
	{
		String m = getClass().getName() + ".release()";
		
		try
		{
			if(getCount() > fMax)
			{
				con.close();
			}
			else
			{
				fConnections.add(con);
			}
		}
		catch(Exception e)
		{
			throw new BDIException(m + " failed to close connection.", e);
		}
		finally
		{
			fLeasedList.remove(con);
		}
	}


	/**
	 * method to close and remove all connections
	 * should be called when finished using the pool
	 *
	 * @throws Exception If the pool could not be destroyed
	 *
	 */
	public void destroy() throws BDIException
	{
		closePool();
	}


	/*
	 * Forces the pool to add a connection.<br>
	 * This  method allows the max limit to be exceeded
	 * and relies on the release method to bring the pool back in line
	 * with the max parameter.
	 *
	 * @throws BDIException if adding a new connection failed
	 *
	 */
	protected void doIncrement() throws BDIException
	{
		String m = getClass().getName() + ".doIncrement()";
		
		try
		{
			int n;

			if(getCount() >= fMax)
			{
				incMonitorMaxReached();
				n = 1;
//				log("doIncrement()", Log.STATUS_WARNING,
//						"Pool: max connections reached or exceeded: count: " + getCount());
			}
			else if((getCount() + getIncrement()) >= fMax)
			{
				n = fMax - getCount();
			}
			else
			{
				n = getIncrement();
			}

			for(int i = 0; i < n; i++)
			{
				addConnection();
			}
		}
		catch(BDIException e)
		{
			throw e;
		}
		catch(Exception e)
		{
			throw new BDIException(m + " failed: ", e);
		}
	}



	/**
	 * Opens the pool
	 *
	 * @throws BDIException If the pool could not be opened
	 *
	 * @see #closePool()
	 *
	 */
	public void openPool() throws BDIException
	{
		setMin(fMin);
	}


	/**
	 * Closes all connections owned by the pool<br>
	 * @note this means those on lease as well
	 *
	 * @see #openPool()
	 */
	public void closePool() throws BDIException
	{
		String m = getClass().getName() + ".closePool()";
		
		for(int i = 0; i < fConnections.size(); i++)
		{
			try
			{
				((Connection) fConnections.get(i)).close();
			}
			catch(Exception e)
			{
				throw new BDIException(m + " close connection: ", e);
			}
		}

		// close all connections in leased bool
		for(int i = 0; i < fLeasedList.size(); i++)
		{
			try
			{
				((Connection) fLeasedList.get(i)).close();
			}
			catch(Exception e)
			{
				throw new BDIException(m + " close connection: ", e);
			}
		}
	}


	/**
	 * Checks a connection
	 *
	 * @return false if the connection is stale
	 *
	 */
	protected boolean checkConnection(Connection con) throws BDIException
	{
		String m = getClass().getName() + ".checkConnection()";
		
		try
		{
			if(con.isClosed())
			{
				incMonitorInvalidConnections();

				return false;
			}
		}
		catch(Exception e)
		{
			throw new BDIException();
		}
		
		return true;
	}


	/**
	 * Gets the monitorInvalidConnections.
	 *
	 * @return the monitorInvalidConnections
	 *
	 */
	public int getMonitorInvalidConnections()
	{
		return fMonitorInvalidConnections;
	}


	/**
	 * Gets the monitorMaxReached.
	 *
	 * @return the monitorMaxReached
	 *
	 */
	public int getMonitorMaxReached()
	{
		return fMonitorMaxReached;
	}


	/**
	 * Increments the monitorInvalidConnections.
	 */
	public void incMonitorInvalidConnections()
	{
		fMonitorInvalidConnections++;
	}


	/**
	 * Increments the monitorMaxReached.
	 */
	public void incMonitorMaxReached()
	{
		fMonitorMaxReached++;
	}

	/**
	 *
	 */
	public int getConnectionsUsedCOunt()
	{
		return fConnectionsUsedCount;
	}

	/**
	 *
	 */
	public void setConnectionsUsedCount(int i)
	{
		fConnectionsUsedCount = i;
	}

	/**
	 *
	 */
	public String getDriver()
	{
		return fConProps.getProperty("driver");
	}

	/**
	 *
	 */
	public String getURL()
	{
		return fConProps.getProperty("url");
	}

	/**
	 *
	 */
	public String getUser()
	{
		return fConProps.getProperty("user");
	}

	/**
	 *
	 */
	public String getPassword()
	{
		return fConProps.getProperty("password");
	}

	/**
	 *
	 */
	public boolean getAutoCommit()
	{
		return Boolean.getBoolean(fConProps.getProperty("autoCommit"));
	}


	/**
	 *
	 */
	public int getMax()
	{
		return fMax;
	}

	/**
	 *
	 */
	public int getIncrement()
	{
		return DEFAULT_INCREMENT_COUNT;
	}

	/**
	 *
	 */
	public int getMin()
	{
		return fMin;
	}

	/**
	 *
	 */
	public void setMax(int max)
	{
		fMax = max;
	}


	/**
	 * Sets the min property.
	 * If the min is greater than the current count
	 * then the difference is made up by
	 * adding connections to the pool.
	 *
	 * @param min The min to set
	 */
	public void setMin(int min) throws BDIException
	{
		fMin = min;

		while(getCount() < min)
			addConnection();
	}

	//---------------------------------------------------------------

	/**
	 * Gets the number of connection, including those on lease
	 *
	 * @return the number of all connections
	 *
	 */
	protected int getCount()
	{
		return fConnections.size() + fLeasedList.size();
	}


	/**
	 * Gets the number of available connection
	 *
	 * @return the number of available connections
	 *
	 */
	public int getAvailableCount()
	{
		return fConnections.size();
	}


	/**
	 * Gets the number of leased connection
	 *
	 * @return the number of avaleasedilable connections
	 *
	 */
	public int getLeasedCount()
	{
		return fLeasedList.size();
	}
	
}
