begin;

-- Create domain tables first

-- Table: "TD_VALIDATION_TYPE"
CREATE TABLE "TD_VALIDATION_TYPE" (
  "VALIDATION_TYPE_ID" int4 NOT NULL, 
  "CODE" varchar(20) NOT NULL, 
  "DESCR" varchar(256), 
  
-- constraints section
  CONSTRAINT "PK_VALIDATION_TYPE_ID" PRIMARY KEY ("VALIDATION_TYPE_ID"),
  CONSTRAINT "UK_VALIDATION_TYPE_CODE" UNIQUE ("CODE")
) WITH OIDS;


-- Table: "TD_DOCUMENT_TYPE"
CREATE TABLE "TD_DOCUMENT_TYPE" (
  "DOCUMENT_TYPE_ID" int4 NOT NULL, 
  "CODE" varchar(64) NOT NULL, 
  "NAME" varchar(64) NOT NULL, 
  "DESCR" varchar(256), 

-- constraints section
  CONSTRAINT "PK_DOCUMENT_TYPE_ID" PRIMARY KEY ("DOCUMENT_TYPE_ID"),
  CONSTRAINT "UK_DOCUMENT_TYPE_CODE" UNIQUE ("CODE"),
  CONSTRAINT "UK_DOCUMENT_TYPE_NAME" UNIQUE ("NAME")
) WITH OIDS;


-- Table: "TD_MESSAGE_TYPE"
CREATE TABLE "TD_MESSAGE_TYPE" (
  "MESSAGE_TYPE_ID" int4 NOT NULL, 
  "CODE" varchar(64) NOT NULL, 
  "DESCR" varchar(256), 

-- constraints section
  CONSTRAINT "PK_MESSAGE_TYPE_ID" PRIMARY KEY ("MESSAGE_TYPE_ID"),
  CONSTRAINT "UK_MESSAGE_TYPE_CODE" UNIQUE ("CODE")
) WITH OIDS;


-- Table: "TD_RELATION_TYPE"
CREATE TABLE "TD_RELATION_TYPE" (
  "RELATION_TYPE_ID" int4 NOT NULL, 
  "CODE" varchar(64) NOT NULL, 
  "DESCR" varchar(256), 

-- constraints section
  CONSTRAINT "PK_RELATION_TYPE_ID" PRIMARY KEY ("RELATION_TYPE_ID"),
  CONSTRAINT "UK_RELATION_TYPE_CODE" UNIQUE ("CODE")
) WITH OIDS;



-- Create other tables

-- Table: "T_TRANSFORMER"
CREATE TABLE "T_TRANSFORMER" (
  "TRANSFORMER_ID" int4 NOT NULL, 
  "JAVA_CLASS_NAME" varchar(128) NOT NULL, 
  "DESCR" varchar(256), 

-- constraints section
  CONSTRAINT "PK_TRANSFORMER_ID" PRIMARY KEY ("TRANSFORMER_ID"),
  CONSTRAINT "UK_JAVA_CLASS_NAME" UNIQUE ("JAVA_CLASS_NAME")
) WITH OIDS;


-- Table: "T_SOURCE_SYSTEM"
CREATE TABLE "T_SOURCE_SYSTEM" (
  "SOURCE_SYSTEM_ID" int4 NOT NULL, 
  "NAME" varchar(128) NOT NULL, 
  "VERSION" varchar(16), 
  "DESCR" varchar(256), 
  "WEB_SITE" varchar(256), 

-- constraints section
  CONSTRAINT "PK_SOURCE_SYSTEM_ID" PRIMARY KEY ("SOURCE_SYSTEM_ID"),
  CONSTRAINT "UK_SOURCE_SYSTEM" UNIQUE ("NAME", "VERSION")
) WITH OIDS;

-- Table: "T_DOCUMENT"
CREATE TABLE "T_DOCUMENT" (
  "DOCUMENT_ID" int4 NOT NULL, 
  "SOURCE_SYSTEM_ID" int4 NOT NULL, 
  "DOCUMENT_TYPE_ID" int4 NOT NULL, 
  "VALIDATION_TYPE_ID" int4 NOT NULL, 
  "VALIDATION_SOURCE" varchar(512), 
  "TRANSFORMER_ID" int4 NOT NULL, 

-- constraints section
  CONSTRAINT "PK_DOCUMENT_ID" PRIMARY KEY ("DOCUMENT_ID"),
  CONSTRAINT "FK_SOURCE_SYSTEM_ID" FOREIGN KEY ("SOURCE_SYSTEM_ID") REFERENCES "T_SOURCE_SYSTEM" ("SOURCE_SYSTEM_ID") ON DELETE CASCADE,
  CONSTRAINT "FK_DOCUMENT_TYPE_ID" FOREIGN KEY ("DOCUMENT_TYPE_ID") REFERENCES "TD_DOCUMENT_TYPE" ("DOCUMENT_TYPE_ID"),
  CONSTRAINT "FK_VALIDATION_TYPE_ID" FOREIGN KEY ("VALIDATION_TYPE_ID") REFERENCES "TD_VALIDATION_TYPE" ("VALIDATION_TYPE_ID"),
  CONSTRAINT "FK_TRANSFORMER_ID" FOREIGN KEY ("TRANSFORMER_ID") REFERENCES "T_TRANSFORMER" ("TRANSFORMER_ID") ON DELETE CASCADE,
  CONSTRAINT "UK_SOURCE_SYSTEM_DOCUMENT_TYPE" UNIQUE ("SOURCE_SYSTEM_ID", "DOCUMENT_TYPE_ID")
) WITH OIDS;

-- Table: "T_MESSAGE"
CREATE TABLE "T_MESSAGE" (
  "MESSAGE_ID" int4 NOT NULL, 
  "DOCUMENT_ID" int4 NOT NULL, 
  "MESSAGE_TYPE_ID" int4 NOT NULL, 
  "MESSAGE_VALUE" text NOT NULL, 
  "DATE_CREATE" timestamp NOT NULL DEFAULT current_timestamp,

-- constraints section
  CONSTRAINT "PK_MESSAGE_ID" PRIMARY KEY ("MESSAGE_ID"),
  CONSTRAINT "FK_DOCUMENT_ID" FOREIGN KEY ("DOCUMENT_ID") REFERENCES "T_DOCUMENT" ("DOCUMENT_ID") ON DELETE CASCADE,
  CONSTRAINT "FK_MESSAGE_TYPE_ID" FOREIGN KEY ("MESSAGE_TYPE_ID") REFERENCES "TD_MESSAGE_TYPE" ("MESSAGE_TYPE_ID")
) WITH OIDS;



-- Table to use as many-to-many relation between messages
-- Table: "TM_MESSAGE_RELATION"
CREATE TABLE "TM_MESSAGE_RELATION" (
  "MESSAGE_RELATION_ID" SERIAL, 
  "PARENT_MESSAGE_ID" int4 NOT NULL, 
  "CHILD_MESSAGE_ID" int4 NOT NULL, 
  "RELATION_TYPE_ID" int4 NOT NULL,

-- constraints section
  CONSTRAINT "PK_MESSAGE_RELATION_ID" PRIMARY KEY ("MESSAGE_RELATION_ID"),
  CONSTRAINT "FK_PARENT_MESSAGE_ID" FOREIGN KEY ("PARENT_MESSAGE_ID") REFERENCES "T_MESSAGE" ("MESSAGE_ID") ON DELETE CASCADE,
  CONSTRAINT "FK_CHILD_MESSAGE_ID" FOREIGN KEY ("CHILD_MESSAGE_ID") REFERENCES "T_MESSAGE" ("MESSAGE_ID") ON DELETE CASCADE,
  CONSTRAINT "FK_RELATION_TYPE_ID" FOREIGN KEY ("RELATION_TYPE_ID") REFERENCES "TD_RELATION_TYPE" ("RELATION_TYPE_ID")
) WITH OIDS;



-- create some addition database objects

-- sequence used for T_MESSAGE.MESSAGE_ID PK population
CREATE SEQUENCE "SEQ_MESSAGE_ID";

-- this procedure takes care of internal details of message creation
CREATE OR REPLACE FUNCTION store_message(INTEGER, INTEGER, TEXT, INTEGER, INTEGER) 
RETURNS INTEGER AS '
DECLARE

	documentId ALIAS FOR $1;
	msgTypeId ALIAS FOR $2;
	msgVal ALIAS FOR $3;

	-- optional, if NULL are passed it means this is a "parent" message
	parentMsgId ALIAS FOR $4;
	relTypeId ALIAS FOR $5;

	newMsgId INTEGER;
BEGIN

	SELECT INTO newMsgId nextval(''public."SEQ_MESSAGE_ID"'');

	INSERT INTO "T_MESSAGE" ("MESSAGE_ID", "DOCUMENT_ID", "MESSAGE_TYPE_ID", "MESSAGE_VALUE")
	VALUES (newMsgId, documentId, msgTypeId, msgVal);

	IF parentMsgId IS NOT NULL and relTypeId IS NOT NULL THEN
		-- insert relationship
		INSERT INTO "TM_MESSAGE_RELATION" ("PARENT_MESSAGE_ID", "CHILD_MESSAGE_ID", "RELATION_TYPE_ID")
		VALUES (parentMsgId, newMsgId, relTypeId);
	END IF;

	RETURN newMsgId;
END;
' LANGUAGE 'plpgsql';


-- as autocommit is set to false the explicit commit must be issued
commit;